# Vite-plugin-cdn-rewrite

适用于 vite 的插件，用于二次处理 vite-plugin-cdn-import 插件的结果

## 原理

- vite-plugin-cdn-import 会将 cdn path 写入 index.html 中。但是我们在内网部署，index.html 的 script 需要的是相对路径，不能携带 ip:port ，所以该插件的作用就是在 rollup 打包的时候将 ip:port 去掉

## 使用说明

1. 安装依赖

```sh
pnpm install vite-plugin-cdn-import @hsmos/vite-plugin-cdn-rewrite -D
```

2. 在 vite.config.ts 中引入依赖

```ts
import importToCDN from 'vite-plugin-cdn-import'
import cdnRewrite, { getPaths } from '@hsmos/vite-plugin-cdn-rewrite'

const devCDNBase = 'http://172.21.44.57:10000'

export default defineConfig({
  plugins: [
    importToCDN({ // 注意此处有顺序要求， vue -> vue-demi -> [other]
      modules: [
        {
          name: 'vue',
          var: 'Vue',
          path: `${devCDNBase}/-/pkg/vue/@3.2.47/vue.global.js`,
        },
        {
          name: 'vue-demi',
          var: 'VueDemi',
          path: `${devCDNBase}/-/pkg/vue-demi/@0.14.6/index.iife.js`,
        },
        {
          name: 'element-plus',
          var: 'ElementPlus',
          path: `${devCDNBase}/-/pkg/element-plus/@2.3.14/index.full.js`,
          css: `${devCDNBase}/-/pkg/element-plus/@2.3.14/index.css`,
        },
      ],
    }),
    // 自定义插件：负责打包后将 index.html 文件中静态资源链接域名部分去掉，方便在内网场景部署
    cdnRewrite() as PluginOption,
  ],
})
```

### 使用注意事项

- index.html head 中一定要有 title 标签，vite-plugin-cdn-import 插入 script 依赖 title 标签；
- importToCDN.modeules 值有顺序要求， element-plus 依赖 vue 和 vue-demi ；
- element-plus 依赖分离之后，就不需要引入 element-plus 的 css 文件了；
- 不要使用 autoImport 插件；
- 提供一个语法糖 getPaths ，可以简化 importToCDN 参数，传入字符串数组或 PkgModel 结构的数组，接收 3 种类型，分别为：

  ```ts
  import importToCDN from 'vite-plugin-cdn-import'
  import { getPaths } from '@hsmos/vite-plugin-cdn-rewrite'

  importToCDN({
      modules: getPaths([
        'vue', // 指定包名，默认加载 依赖池 中的默认版本
        {
          name: 'vue-demi',
          version: '@0.14.6', // 指定版本，当 version 和 path 参数同时存在时，以 version 为准
        },
        {
          name: 'element-plus',
          var: 'ElementPlus', // 使用 path 时，var 属性必填，全局变量名，path 的域必须为 http://172.21.44.57:10000
          path: `http://172.21.44.57:10000/-/pkg/element-plus/@2.3.14/index.full.js`,
          css: `http://172.21.44.57:10000/-/pkg/element-plus/@2.3.14/index.css`,
        }
      ])
  })
  ```

## 类型说明

```ts
/**
 * getPaths 入参和返回结构
 */
export interface PkgModel {
    name: string;
    var: string;
    path: string | string[];
    css?: string;
    version?: string;
}

/**
 * 获取 importToCDN.modules 参数
 * @param pkg 字符串 PkgModel 类型的对象数组
 * @returns PkgModel[]
 */
export declare function getPaths(pkg: Array<string | PkgModel>): Array<PkgModel>;

/**
 * 重写 cdn path 插件
 * @returns
 */
export default function cdnRewrite(): {
    name: string;
    apply: string;
    writeBundle(options: any, bundle: any): void;
};
```

## 优缺点分析

### 优点

- 统一生产环境依赖版本；
- 提高打包速度；
- 提高网络请求资源复用率；
- 减轻服务器压力；

### 缺点

- CDN 比较适合公网环境，在内网使用需要部署多套静态资源服务，运维成本高；
- 不同构建工具实现依赖分离打包方式不同，开发成本高；
- 不能完全解决开发环境版本一致问题，注意开发环境最好使用与 cdn 版本一致的依赖；
- 可能因开发环境和生产环境版本不一致造成线上问题；
- 依赖分离和按需加载只能选其一；

## 依赖池清单

依赖包资源可通过 <http://172.21.44.57:10000/-/pkg/> 访问

包名|var|版本
--|--|--
vue           |Vue          |@3.2.47(default)
vue-demi      |VueDemi      |@0.14.6(default)
element-plus  |ElementPlus  |@2.3.14(default)
vue-router    |VueRouter    |@4.1.6(default)
pinia         |Pinia        |@2.0.30(default)
dayjs         |dayjs        |@0.11.10(default)
validator     |validator    |@13.9.0(default)
echarts       |echarts      |@5.4.3(default)

## 更新日志

- v0.1.0 - 2023-09-28
  - vite-plugin-cdn-rewrite 插件发布；
- v0.1.1 - 2023-10-07
  - 新增 vue-router@4.1.6 和 pinia@2.0.30；
- v0.1.2 - 2023-10-19
  - 新增 dayjs@0.11.10 、 validator@13.9.0 、echarts@5.4.3；
