import { join } from 'node:path'
import { writeFileSync } from 'node:fs'

export interface PkgModel {
  name: string
  var: string
  path: string | string[]
  css?: string
  version?: string
}

interface PkgPool {
  name: string
  var: string
  version: Record<string, {
    path: string
    css?: string
    default?: boolean
  }>
}

const devCDNBase = 'http://172.21.44.57:10000'

/**
 * cdn 池
 */
export const pkgPool: PkgPool[] = [
  {
    name: 'vue',
    var: 'Vue',
    version: {
      '@3.2.47': {
        path: `${devCDNBase}/-/pkg/vue/@3.2.47/vue.global.js`,
        default: true
      }
    }
  },
  {
    name: 'vue-demi',
    var: 'VueDemi',
    version: {
      '@0.14.6': {
        path: `${devCDNBase}/-/pkg/vue-demi/@0.14.6/index.iife.js`,
        default: true
      }
    }
  },
  {
    name: 'element-plus',
    var: 'ElementPlus',
    version: {
      '@2.3.14': {
        path: `${devCDNBase}/-/pkg/element-plus/@2.3.14/index.full.js`,
        css: `${devCDNBase}/-/pkg/element-plus/@2.3.14/index.css`,
        default: true
      }
    }
  },
  {
    name: 'vue-router',
    var: 'VueRouter',
    version: {
      '@4.1.6': {
        path: `${devCDNBase}/-/pkg/vue-router/@4.1.6/vue-router.global.js`,
        default: true
      }
    }
  },
  {
    name: 'pinia',
    var: 'Pinia',
    version: {
      '@2.0.30': {
        path: `${devCDNBase}/-/pkg/pinia/@2.0.30/pinia.iife.js`,
        default: true
      }
    }
  },
  {
    name: 'dayjs',
    var: 'dayjs',
    version: {
      '@0.11.10': {
          path: `${devCDNBase}/-/pkg/dayjs/@0.11.10/dayjs.min.js`,
          default: true
      }
    }
  },
  {
      name: 'validator',
      var: 'validator',
      version: {
        '@13.9.0': {
            path: `${devCDNBase}/-/pkg/validator/@13.9.0/validator.min.js`,
            default: true
        }
      }
  },
  {
    name: 'echarts',
    var: 'echarts',
    version: {
      '@5.4.3': {
          path: `${devCDNBase}/-/pkg/echarts/@5.4.3/echarts.min.js`,
          default: true
      }
    }
  },
]

export function getAllPkg() {
  return pkgPool.map(v => v.name)
}

/**
 * 获取 importToCDN.modules 参数
 * @param pkg 字符串 PkgModel 类型的对象数组
 * @returns PkgModel[]
 */
export function getPaths(pkg: Array<string | PkgModel>): Array<PkgModel> {
  const res: PkgModel[] = []
  if(pkg?.length) {
    pkg.forEach(p => {
      const getPack = getPkgModel(p)
      if(getPack) {
        res.push(getPack)
      }
    })
  }
  return res
}

// 从 pool 中获取一个 PkgModel 
function getPkgModel(pkg: string | PkgModel) {
  let res: PkgModel | undefined
  if(typeof pkg === 'string') {
    const pack = pkgPool.find(v => v.name === pkg)
    if(pack) {
      res =  {
        name: pack.name,
        var: pack.var,
        path: ''
      }
      for(const version in pack.version) {
        if(pack.version[version]?.default) {
          const defaultVersion = pack.version[version]
          if(defaultVersion?.path) {
            res.path = defaultVersion.path
          }
          if(defaultVersion?.css) {
            res.css = defaultVersion.css
          }
        }
      }
    } else {
      console.error(`[vite-plugin-cdn-rewrite] ${pkg} not in pkgPool`)
    }
  } else {
    if(pkg.name) {
      if(pkg.version) {
        // 从 pool 中找 name
        const pack = pkgPool.find(v => v.name === pkg.name)
        if(pack) {
          res =  {
            name: pack.name,
            var: pack.var,
            path: ''
          }
          const targetVersion = pack.version[pkg.version]
          if(targetVersion) {
              if(targetVersion?.path) {
                res.path = targetVersion.path
              }
              if(targetVersion?.css) {
                res.css = targetVersion.css
              }
          } else {
            console.error(`[vite-plugin-cdn-rewrite] ${pkg.name}'s version ${pkg.version} not in pkgPool`)
          }
        } else {
          console.error(`[vite-plugin-cdn-rewrite] ${pkg.name} not in pkgPool`)
        }
      } else if(pkg.var && (pkg.path || pkg.css)) {
        res = {
          name: pkg.name,
          var: pkg.var,
          path: ''
        }
        if(pkg.path) {
          res.path = pkg.path
        }
        if(pkg.css) {
          res.css = pkg.css
        }
      }
    } else {
      console.error(`[vite-plugin-cdn-rewrite] Please check pkg name`)
    }
  }
  return res
}

/**
 * 重写 cdn path 插件
 * @returns 
 */
export default function cdnRewrite () {
  return {
    name: 'vite:set-index-base',
    apply: 'build',
    writeBundle(options: any, bundle: any) {
      for (const chunkName in bundle) {
        if (
          Object.prototype.hasOwnProperty.call(bundle, chunkName) &&
          chunkName === 'index.html'
        ) {
          const chunk: any = bundle[chunkName]
          if (chunk.source && options.dir) {
            chunk.source = chunk.source.replace(
              new RegExp(devCDNBase, 'g'),
              ''
            )
            const fullPath = join(options.dir, chunk.fileName)
            writeFileSync(fullPath, chunk.source)
          }
        }
      }
    },
  }
}
